#include <math.h>
#include <stdlib.h>

#include "cctk.h"
#include "cctk_Parameters.h"
#include "cctk_Arguments.h"


#define PI 3.141592653589793238462643383279502884197169399375105820974944592307816406286208998628034825342117068

#define velx         (&vel[0*cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]])
#define vely         (&vel[1*cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]])
#define velz         (&vel[2*cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]])
#define velx_p     (&vel_p[0*cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]])
#define vely_p     (&vel_p[1*cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]])
#define velz_p     (&vel_p[2*cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]])
#define velx_p_p (&vel_p_p[0*cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]])
#define vely_p_p (&vel_p_p[1*cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]])
#define velz_p_p (&vel_p_p[2*cctk_lsh[0]*cctk_lsh[1]*cctk_lsh[2]])



extern"C" void
Perturb_Velocity(CCTK_ARGUMENTS)
{
  DECLARE_CCTK_ARGUMENTS;
  DECLARE_CCTK_PARAMETERS;

  for (int k=0; k < cctk_lsh[2]; ++k)
     for (int j=0; j < cctk_lsh[1]; ++j)
        for (int i=0; i < cctk_lsh[0]; ++i)

    {
       const int ijk = CCTK_GFINDEX3D(cctkGH, i,j,k);
       
       //determine if we're in the region we want to perturb
       const CCTK_REAL cyl_r   = sqrt(x[ijk]*x[ijk] + y[ijk]*y[ijk]);
       if( cyl_r>=cyl_r_min && cyl_r<=cyl_r_max && fabs(z[ijk])>=z_min && fabs(z[ijk])<z_max ){
 
	 // get this grid cell's total velocity
	 const CCTK_REAL tot_vel     = sqrt(velx[ijk]*velx[ijk]         + vely[ijk]*vely[ijk]         + velz[ijk]*velz[ijk]);
	 const CCTK_REAL tot_vel_p   = sqrt(velx_p[ijk]*velx_p[ijk]     + vely_p[ijk]*vely_p[ijk]     + velz_p[ijk]*velz_p[ijk]);
	 const CCTK_REAL tot_vel_p_p = sqrt(velx_p_p[ijk]*velx_p_p[ijk] + vely_p_p[ijk]*vely_p_p[ijk] + velz_p_p[ijk]*velz_p_p[ijk]);
	 
	 // determine the angle from the vertical wavenumber
	 const CCTK_REAL cyl_phi = (m1_same_handedness ? (k_z * fabs(z[ijk])) : (k_z * z[ijk]) );
	 
	 // Perturb the velocities
	 if (pert_vel_random){
	   velx[ijk]     += 2.0 * ((double)rand()/(double)RAND_MAX - 0.5) * tot_vel     * vel_pert_amp;
	   vely[ijk]     += 2.0 * ((double)rand()/(double)RAND_MAX - 0.5) * tot_vel     * vel_pert_amp;
//	   velz[ijk]     += 2.0 * ((double)rand()/(double)RAND_MAX - 0.5) * tot_vel     * vel_pert_amp;
	   velx_p[ijk]   += 2.0 * ((double)rand()/(double)RAND_MAX - 0.5) * tot_vel_p   * vel_pert_amp;
	   vely_p[ijk]   += 2.0 * ((double)rand()/(double)RAND_MAX - 0.5) * tot_vel_p   * vel_pert_amp;
//	   velz_p[ijk]   += 2.0 * ((double)rand()/(double)RAND_MAX - 0.5) * tot_vel_p   * vel_pert_amp;
	   velx_p_p[ijk] += 2.0 * ((double)rand()/(double)RAND_MAX - 0.5) * tot_vel_p_p * vel_pert_amp;
	   vely_p_p[ijk] += 2.0 * ((double)rand()/(double)RAND_MAX - 0.5) * tot_vel_p_p * vel_pert_amp;
//	   velz_p_p[ijk] += 2.0 * ((double)rand()/(double)RAND_MAX - 0.5) * tot_vel_p_p * vel_pert_amp;
	 }
	 if(pert_vel_m1){
	   CCTK_REAL velxt =  cos(cyl_phi) * tot_vel     * vel_pert_amp;
	   CCTK_REAL velyt = sin(cyl_phi) * tot_vel     * vel_pert_amp;
	   velx[ijk]     += cos(cyl_phi) * tot_vel     * vel_pert_amp;
	   vely[ijk]     += sin(cyl_phi) * tot_vel     * vel_pert_amp;
	   velx_p[ijk]   += cos(cyl_phi) * tot_vel_p   * vel_pert_amp;
	   vely_p[ijk]   += sin(cyl_phi) * tot_vel_p   * vel_pert_amp;
	   velx_p_p[ijk] += cos(cyl_phi) * tot_vel_p_p * vel_pert_amp;
	   vely_p_p[ijk] += sin(cyl_phi) * tot_vel_p_p * vel_pert_amp;
           CCTK_WARN(1,"Inside velocity perturbation vel_1");
           CCTK_VWarn(1, __LINE__, __FILE__, CCTK_THORNSTRING,
                       "vxt,vyt: %15.6E %15.6E", velxt, velyt);
	 }
       }
    }

  return;
}



